

/*
 * The JTS Topology Suite is a collection of Java classes that
 * implement the fundamental operations required to validate a given
 * geo-spatial data set to a known topological specification.
 *
 * Copyright (C) 2001 Vivid Solutions
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more information, contact:
 *
 *     Vivid Solutions
 *     Suite #1A
 *     2328 Government Street
 *     Victoria BC  V8T 5G5
 *     Canada
 *
 *     (250)385-6040
 *     www.vividsolutions.com
 */
package com.vividsolutions.jts.algorithm;

/**
 *@version 1.4.0
 */

import com.vividsolutions.jts.geom.DefaultCoordinateSequenceFactory;
import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.CoordinateSequence;
import com.vividsolutions.jts.geom.Envelope;

/**
 * Robust versions of various fundamental Computational Geometric algorithms.
 * The algorithms are made robust by use of the {@link RobustDeterminant} class.
 *
 * @version 1.4.0
 */
public class RobustCGAlgorithms extends CGAlgorithms {

  /**
   * Returns the index of the direction of the point <code>q</code>
   * relative to a
   * vector specified by <code>p1-p2</code>.
   *
   * @param p1 the origin point of the vector
   * @param p2 the final point of the vector
   * @param q the point to compute the direction to
   *
   * @return 1 if q is counter-clockwise (left) from p1-p2
   * @return -1 if q is clockwise (right) from p1-p2
   * @return 0 if q is collinear with p1-p2
   */
  public static int orientationIndex(Coordinate p1, Coordinate p2, Coordinate q) {
    // travelling along p1->p2, turn counter clockwise to get to q return 1,
    // travelling along p1->p2, turn clockwise to get to q return -1,
    // p1, p2 and q are colinear return 0.
    double dx1 = p2.x - p1.x;
    double dy1 = p2.y - p1.y;
    double dx2 = q.x - p2.x;
    double dy2 = q.y - p2.y;
    return RobustDeterminant.signOfDet2x2(dx1, dy1, dx2, dy2);
  }

  /**
   * Computes whether a ring defined by an array of {@link Coordinate} is
   * oriented counter-clockwise.
   * <p>
   * This algorithm is valid only for coordinate lists which do not contain
   * repeated points.
   *
   * @param ring an array of coordinates forming a ring
   * @return <code>true</code> if the ring is oriented counter-clockwise.
   */
  public boolean isCCW(Coordinate[] ring) {
    Coordinate hip;
    Coordinate p;
    Coordinate prev;
    Coordinate next;
    int hii;
    int i;
    int nPts = ring.length;

    // check that this is a valid ring - if not, simply return a dummy value
    if (nPts < 4) return false;

    // algorithm to check if a Ring is stored in CCW order
    // find highest point
    hip = ring[0];
    hii = 0;
    for (i = 1; i < nPts; i++) {
      p = ring[i];
      if (p.y > hip.y) {
        hip = p;
        hii = i;
      }
    }
    // find points on either side of highest
    int iPrev = hii - 1;
    if (iPrev < 0) {
      iPrev = nPts - 2;
    }
    int iNext = hii + 1;
    if (iNext >= nPts) {
      iNext = 1;
    }
    prev = ring[iPrev];
    next = ring[iNext];
    int disc = computeOrientation(prev, hip, next);
    /*
     *  If disc is exactly 0, lines are collinear.  There are two possible cases:
     *  (1) the lines lie along the x axis in opposite directions
     *  (2) the line lie on top of one another
     *  (2) should never happen, so we're going to ignore it!
     *  (Might want to assert this)
     *  (1) is handled by checking if next is left of prev ==> CCW
     */
    if (disc == 0) {
      // poly is CCW if prev x is right of next x
      return (prev.x > next.x);
    }
    else {
      // if area is positive, points are ordered CCW
      return (disc > 0);
    }

  }

  /**
   * This algorithm does not attempt to first check the point against the envelope
   * of the ring.
   *
   * @param ring assumed to have first point identical to last point
   */
  public boolean isPointInRing(Coordinate p, Coordinate[] ring) {
    int i;
    int i1;       // point index; i1 = i-1
    double xInt;  // x intersection of segment with ray
    int crossings = 0;  // number of segment/ray crossings
    double x1;    // translated coordinates
    double y1;
    double x2;
    double y2;
    int nPts = ring.length;

    /*
     *  For each segment l = (i-1, i), see if it crosses ray from test point in positive x direction.
     */
    for (i = 1; i < nPts; i++) {
      i1 = i - 1;
      Coordinate p1 = ring[i];
      Coordinate p2 = ring[i1];
      x1 = p1.x - p.x;
      y1 = p1.y - p.y;
      x2 = p2.x - p.x;
      y2 = p2.y - p.y;

      if (((y1 > 0) && (y2 <= 0)) ||
          ((y2 > 0) && (y1 <= 0))) {
        /*
         *  segment straddles x axis, so compute intersection.
         */
        xInt = RobustDeterminant.signOfDet2x2(x1, y1, x2, y2) / (y2 - y1);
        //xsave = xInt;
        /*
         *  crosses ray if strictly positive intersection.
         */
        if (0.0 < xInt) {
          crossings++;
        }
      }
    }
    /*
     *  p is inside if number of crossings is odd.
     */
    if ((crossings % 2) == 1) {
      return true;
    }
    else {
      return false;
    }
  }

  public boolean isOnLine(Coordinate p, Coordinate[] pt) {
    LineIntersector lineIntersector = new RobustLineIntersector();
    for (int i = 1; i < pt.length; i++) {
      Coordinate p0 = pt[i - 1];
      Coordinate p1 = pt[i];
      lineIntersector.computeIntersection(p, p0, p1);
      if (lineIntersector.hasIntersection()) {
        return true;
      }
    }
    return false;
  }

  public int computeOrientation(Coordinate p1, Coordinate p2, Coordinate q) {
    return orientationIndex(p1, p2, q);
  }

  private boolean isInEnvelope(Coordinate p, Coordinate[] ring) {
    Envelope envelope = new Envelope();
    for (int i = 0; i < ring.length; i++) {
      envelope.expandToInclude(ring[i]);
    }
    return envelope.contains(p);
  }

}
