
/*
 * The JTS Topology Suite is a collection of Java classes that
 * implement the fundamental operations required to validate a given
 * geo-spatial data set to a known topological specification.
 *
 * Copyright (C) 2001 Vivid Solutions
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more information, contact:
 *
 *     Vivid Solutions
 *     Suite #1A
 *     2328 Government Street
 *     Victoria BC  V8T 5G5
 *     Canada
 *
 *     (250)385-6040
 *     www.vividsolutions.com
 */
package com.vividsolutions.jts.geom;

/**
 * The internal representation of a list of coordinates inside a Geometry.
 * <p>
 * There are some cases in which you might want Geometries to store their
 * points using something other than the JTS Coordinate class. For example, you
 * may want to experiment with another implementation, such as an array of xs
 * and an array of ys. or you might want to use your own coordinate class, one
 * that supports extra attributes like M-values.
 * <p>
 * You can do this by implementing the CoordinateSequence and
 * CoordinateSequenceFactory interfaces. You would then create a
 * GeometryFactory parameterized by your CoordinateSequenceFactory, and use
 * this GeometryFactory to create new Geometries. All of these new Geometries
 * will use your CoordinateSequence implementation.
 * <p>
 * For an example, see the code for
 * {@link com.vividsolutions.jtsexample.geom.TwoArrayCoordinateSequenceExample}.
 * <p>
 * A note on performance. If your CoordinateSequence is not based on an array
 * of Coordinates, it may incur a performance penalty when its #toArray method
 * is called because the array needs to be built from scratch.
 * 
 * @see DefaultCoordinateSequenceFactory
 * @see TwoArrayCoordinateSequenceFactory
 *
 * @version 1.4.0
 */
public interface CoordinateSequence extends Cloneable {   
    
    /**
     * Returns (possibly a copy of) the ith Coordinate in this collection.
     * Whether or not the Coordinate returned is the actual underlying
     * Coordinate or merely a copy depends on the implementation.
     * <p>
     * Note that in the future the semantics of this method may change
     * to guarantee that the Coordinate returned is always a copy. Callers are
     * advised not to assume that they can modify a CoordinateSequence by
     * modifying the Coordinate returned by this method.
     */
    public Coordinate getCoordinate(int i);

    /**
     * Returns the number of Coordinates (actual or
     * otherwise, as this implementation may not store its data in Coordinate
     * objects). 
     */
    public int size();
    
    /**
     * Returns (possibly copies of) the Coordinates in this collection.
     * Whether or not the Coordinates returned are the actual underlying
     * Coordinates or merely copies depends on the implementation. Note that
     * if this implementation does not store its data as an array of Coordinates,
     * this method will incur a performance penalty because the array needs to
     * be built from scratch.
     */
    public Coordinate[] toCoordinateArray();
    
    /**
     * Returns a deep copy of this collection. Called by Geometry#clone.
     */
    public Object clone();
        
}
