
/*
 * The JTS Topology Suite is a collection of Java classes that
 * implement the fundamental operations required to validate a given
 * geo-spatial data set to a known topological specification.
 *
 * Copyright (C) 2001 Vivid Solutions
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more information, contact:
 *
 *     Vivid Solutions
 *     Suite #1A
 *     2328 Government Street
 *     Victoria BC  V8T 5G5
 *     Canada
 *
 *     (250)385-6040
 *     www.vividsolutions.com
 */
package com.vividsolutions.jtsexample.geom;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.CoordinateSequence;
import com.vividsolutions.jts.geom.CoordinateSequenceFactory;

/**
 * Creates CoordinateSequences internally represented as an array of x's and
 * an array of y's.
 *
 * @version 1.4.0
 */
public class TwoArrayCoordinateSequenceFactory implements CoordinateSequenceFactory {
    private static TwoArrayCoordinateSequenceFactory instance = new TwoArrayCoordinateSequenceFactory();
	private TwoArrayCoordinateSequenceFactory() {
    }

    /**
     * Returns the singleton instance of TwoArrayCoordinateSequenceFactory
     */
    public static TwoArrayCoordinateSequenceFactory instance() {
        return instance;
    }    

    public CoordinateSequence create(Coordinate[] coordinates) {
        return new TwoArrayCoordinateSequence(coordinates);
    }    

    private static class TwoArrayCoordinateSequence implements CoordinateSequence {
        private double[] xs;
        private double[] ys;

        public TwoArrayCoordinateSequence(Coordinate[] coordinates) {            
            xs = new double[coordinates.length];
            ys = new double[coordinates.length];
            for (int i = 0; i < coordinates.length; i++) {
                xs[i] = coordinates[i].x;
                ys[i] = coordinates[i].y;
            }
        }        

        public Coordinate getCoordinate(int i) {
			return new Coordinate(xs[i], ys[i]);
		}                
        
        public Object clone() {
            double[] xsClone = new double[xs.length];
            double[] ysClone = new double[ys.length];
            System.arraycopy(xs, 0, xsClone, 0, xs.length);
            System.arraycopy(ys, 0, ysClone, 0, ys.length);
            return new TwoArrayCoordinateSequence(xsClone, ysClone);
        }

        public int size() {
            return xs.length;
        }

        public Coordinate[] toCoordinateArray() {
            Coordinate[] array = new Coordinate[xs.length];
            for (int i = 0; i < array.length; i++) {
                array[i] = new Coordinate(xs[i], ys[i]);
            }
            return array;
        }

		private TwoArrayCoordinateSequence(double[] xs, double[] ys) {            
		    this.xs = xs;
		    this.ys = ys;
		}
        
        public String toString()
        {
          StringBuffer strBuf = new StringBuffer();
          strBuf.append("TwoArrayCoordinateSequence [");
          for (int i = 0; i < size(); i++) {
            if (i > 0) strBuf.append(", ");
            strBuf.append(xs[i] + " " + ys[i]);
          }
          strBuf.append("]");
          return strBuf.toString();
        }        

    }

}
